

//-----------------------------------------------------------------------------
// Copyright 2018 Peter Balch
// Modified by Przemyslaw Wodz to use as a signal source for Curve Tracer
//-----------------------------------------------------------------------------

#include <math.h>
#include <Wire.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_GFX.h>
#include <SimpleRotary.h>
#define OLED_ADDR   0x3C
#define SCREEN_WIDTH 128                                      // definicja wymiarow wyswietlacza - znacznie przyspiesza odswiezanie ekranu:)
#define SCREEN_HEIGHT 64
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT);

SimpleRotary rotary(6,5,7);


//-----------------------------------------------------------------------------
// Zmienne globalne
//-----------------------------------------------------------------------------

const int CS = 10;
const int SG_fsyncPin = 2;
const int SG_CLK = 3;
const int SG_DATA = 4;
const int wSine     = 0b0000000000000000;
int waveType = wSine;
int wave;
int freqSet;
int voltage;
int level;
int zm2;
long frequency;
long zm1;
int volume;




void setup (void) {

   pinMode (CS, OUTPUT);
   SPI.begin(); // initialize SPI
   display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
   voltage = 3;
   freqSet = 2;
   InitSigGen();
   }
 
  void checkFreq() {
 
   if (freqSet == 1)
  {
    frequency = 50;                             // czestotliwosc
    volume = 200;                               // nastawa dla MCP41010
  }
  if (freqSet== 2)
  {
    frequency = 500;
    volume = 200;
  }
  if (freqSet == 3)
  {
    frequency = 2000;
    volume = 200;
  }
  if (freqSet == 4)
  {
    frequency = 10000;
    volume = 200;
  }
  if (freqSet == 5)
  {
    frequency = 50000;
    volume = 255;                             // nastawa dla MCP41010 - wieksza bo spadek wzmocnienia przy tej czestotliwosci
  }

}

//-----------------------------------------------------------------------------
// SG_WriteRegister
//-----------------------------------------------------------------------------
void SG_WriteRegister(word dat) {
      digitalWrite(SG_CLK, LOW);
      digitalWrite(SG_CLK, HIGH);
      digitalWrite(SG_fsyncPin, LOW);
  for (byte i = 0; i < 16; i++) {
    if (dat & 0x8000)
      digitalWrite(SG_DATA, HIGH);
    else
      digitalWrite(SG_DATA, LOW);
    dat = dat << 1;
      digitalWrite(SG_CLK, HIGH);
      digitalWrite(SG_CLK, LOW);
  }
      digitalWrite(SG_CLK, HIGH);
      digitalWrite(SG_fsyncPin, HIGH);
}

//-----------------------------------------------------------------------------
// SG_Reset
//-----------------------------------------------------------------------------
void SG_Reset() {
  delay(100);
  SG_WriteRegister(0x100);
  delay(100);
}

void InitSigGen(void) {

  pinMode(SG_DATA, OUTPUT);
  pinMode(SG_CLK, OUTPUT);
  pinMode(SG_fsyncPin, OUTPUT);
  digitalWrite(SG_fsyncPin, HIGH);
  digitalWrite(SG_CLK, HIGH);
  SG_Reset();

}


//-----------------------------------------------------------------------------
// Main routines
// loop
//-----------------------------------------------------------------------------
 void loop (void) {

 checkFreq();
 
 byte i;
i = rotary.push();
   if ( i == 1 ) {
    freqSet = freqSet+1;
    if (freqSet>5) {freqSet = 1;}
   }
   
byte k;
k = rotary.rotate();

 if (k ==1) {
  voltage = voltage +1;
  if (voltage>26) {voltage = 26;}
 }

 if (k==2) {
  voltage = voltage-1;
  if (voltage<0) {voltage = 0;}
 }

level = map(voltage,0,26,0,volume);

  

 if (zm1!= frequency){

  digitalWrite(CS,LOW); // select the chip
   SPI.transfer(B00010001); // command byte
   SPI.transfer(level); // data byte 
   digitalWrite(CS,HIGH); // de-select the chip
   
   display.clearDisplay();
   display.setTextSize(1);
   display.setCursor(5,3);
   display.setTextColor(WHITE);
   display.drawRect(1, 1, 127, 12, SSD1306_WHITE);
   display.println("* COMPONENT TRACER *");
   display.setCursor(25, 16 );
   display.println("FREQ / VOLTAGE" ); 
   display.setCursor(15,30 );
   display.setTextSize(2);
   display.println(frequency );
   display.setCursor(90, 30 );
   display.println("Hz" );
   display.setCursor(15,50 );
   display.setTextSize(2);
   display.println(voltage );
   display.setCursor(90, 50 );
   display.println("Vpp" );
   display.display();
  

 // void SG_freqReset() 
  long fl = frequency * (0x10000000 / 25000000.0);
  SG_WriteRegister(0x2100);
  SG_WriteRegister((int)(fl & 0x3FFF) | 0x4000);
  SG_WriteRegister((int)((fl & 0xFFFC000) >> 14) | 0x4000);
  SG_WriteRegister(0xC000);
  SG_WriteRegister(wave);
  waveType = wave;

//void SG_freqSet( ) 
  SG_WriteRegister(0x2000 | wave);
  SG_WriteRegister((int)(fl & 0x3FFF) | 0x4000);
  SG_WriteRegister((int)((fl & 0xFFFC000) >> 14) | 0x4000);

  }

  if (zm2!= voltage){

   digitalWrite(CS,LOW); // select the chip
   SPI.transfer(B00010001); // command byte
   SPI.transfer(level); // data byte 
   digitalWrite(CS,HIGH); // de-select the chip

    
   display.clearDisplay();
   display.setTextSize(1);
   display.setCursor(5,3);
   display.setTextColor(WHITE);
   display.drawRect(1, 1, 127, 12, SSD1306_WHITE);
   display.println("* COMPONENT TRACER *");
   display.setCursor(25, 16 );
   display.println("FREQ / VOLTAGE" ); 
   display.setCursor(15,30 );
   display.setTextSize(2);
   display.println(frequency );
   display.setCursor(90, 30 );
   display.println("Hz" );
   display.setCursor(15,50 );
   display.setTextSize(2);
   display.println(voltage );
   display.setCursor(90, 50 );
   display.println("Vpp" );
   display.display();
  }
  
zm1= frequency;
zm2 = voltage;
}
